package ink.wgink.module.wps.controller.api;

import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.wps.consts.IWpsConsts;
import ink.wgink.module.wps.pojo.dtos.WpsFileInfoDTO;
import ink.wgink.module.wps.pojo.dtos.WpsSaveFileInfoDTO;
import ink.wgink.module.wps.service.IWpsService;
import ink.wgink.pojo.result.ErrorResult;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * @ClassName: WpsController
 * @Description: WPS
 * @Author: wanggeng
 * @Date: 2022/6/22 09:22
 * @Version: 1.0
 */
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "wps")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/wps")
public class WpsController {

    @Autowired
    private IWpsService wpsService;

    @ApiOperation(value = "保存文件", notes = "保存文件接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "_w_fileid", value = "文件ID", paramType = "form"),
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("v1/3rd/file/save")
    public WpsSaveFileInfoDTO save(@RequestHeader(IWpsConsts.HEADER_WPS_WEB_OFFICE_TOKEN) String wpsToken, @RequestParam(IWpsConsts.KEY_FILE_ID) String fileId, @RequestParam("file") MultipartFile file) {
        return wpsService.save(wpsToken, fileId, file);
    }

    @ApiOperation(value = "文件详情", notes = "文件详情接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "_w_fileid", value = "文件ID", paramType = "query")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("v1/3rd/file/info")
    public WpsFileInfoDTO getFileInfo(@RequestHeader(IWpsConsts.HEADER_WPS_WEB_OFFICE_TOKEN) String wpsToken, @RequestParam(IWpsConsts.KEY_FILE_ID) String fileId) {
        return wpsService.getFileInfo(wpsToken, fileId);
    }



    @ApiOperation(value = "文件下载", notes = "文件下载接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "fileId", value = "文件ID", paramType = "query")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("download/{fileId}")
    public ResponseEntity<byte[]> download(@PathVariable("fileId") String fileId) throws Exception {
        return wpsService.download(fileId);
    }

}
