package ink.wgink.module.wps.controller.route;

import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.wps.enums.WpsPermissionEnum;
import ink.wgink.module.wps.pojo.bos.WpsTokenBO;
import ink.wgink.module.wps.service.IWpsService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import java.io.UnsupportedEncodingException;

/**
 * @ClassName: WpsRouteController
 * @Description: wps
 * @Author: wanggeng
 * @Date: 2022/6/21 23:30
 * @Version: 1.0
 */
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "wps路由接口")
@Controller
@RequestMapping(ISystemConstant.ROUTE_PREFIX + "/wps")
public class WpsRouteController {

    @Autowired
    private IWpsService wpsService;

    @GetMapping("get/{fileId}")
    public ModelAndView get(@PathVariable("fileId") String fileId) throws UnsupportedEncodingException {
        ModelAndView mv = getBaseMV(WpsPermissionEnum.READ, fileId);
        mv.setViewName("wps/wps");
        return mv;
    }

    @GetMapping("update/{fileId}")
    public ModelAndView update(@PathVariable("fileId") String fileId) throws UnsupportedEncodingException {
        ModelAndView mv = getBaseMV(WpsPermissionEnum.WRITE, fileId);
        mv.setViewName("wps/wps");
        return mv;
    }

    /**
     * 获取基础ModelAndView
     *
     * @param fileId
     * @return
     * @throws UnsupportedEncodingException
     */
    private ModelAndView getBaseMV(WpsPermissionEnum wpsPermissionEnum, String fileId) throws UnsupportedEncodingException {
        WpsTokenBO wpsTokenBO = wpsService.getToken(wpsPermissionEnum, fileId);
        ModelAndView mv = new ModelAndView();
        mv.addObject("wpsUrl", wpsTokenBO.getWpsUrl());
        mv.addObject("token", wpsTokenBO.getToken());
        return mv;
    }

}
