package ink.wgink.module.wps.service.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.interfaces.user.IUserBaseService;
import ink.wgink.module.file.service.v2.IFileV2Service;
import ink.wgink.module.wps.consts.IWpsConsts;
import ink.wgink.module.wps.enums.WpsPermissionEnum;
import ink.wgink.module.wps.pojo.bos.WpsTokenBO;
import ink.wgink.module.wps.pojo.dtos.WpsFileInfoDTO;
import ink.wgink.module.wps.pojo.dtos.WpsSaveFileInfoDTO;
import ink.wgink.module.wps.service.IWpsService;
import ink.wgink.module.wps.util.WpsUtil;
import ink.wgink.pojo.dtos.user.UserDTO;
import ink.wgink.pojo.pos.FilePO;
import ink.wgink.properties.FileProperties;
import ink.wgink.properties.ServerProperties;
import ink.wgink.properties.wps.WpsProperties;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

/**
 * @ClassName: WpsServiceImpl
 * @Description: wps
 * @Author: wanggeng
 * @Date: 2022/6/21 21:47
 * @Version: 1.0
 */
@Service
public class WpsServiceImpl extends DefaultBaseService implements IWpsService {

    @Autowired
    private WpsProperties wpsProperties;
    @Autowired
    private FileProperties fileProperties;
    @Autowired
    private IFileV2Service fileV2Service;
    @Autowired
    private IUserBaseService userBaseService;
    @Autowired
    private ServerProperties serverProperties;

    @Override
    public WpsTokenBO getToken(WpsPermissionEnum wpsPermissionEnum, String fileId) throws UnsupportedEncodingException {
        FilePO filePO = fileV2Service.getPO(fileId);
        if (filePO == null) {
            throw new SearchException("文件不存在");
        }
        if (!WpsUtil.isWpsFile(filePO.getFileType())) {
            throw new SearchException("文件格式不支持");
        }
        String userId = securityComponent.getCurrentUser().getUserId();
        String wpsUrl = wpsProperties.getUrl() + "/office/" + WpsUtil.getWpsType(filePO.getFileType()) + "/" + fileId + "?";
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put(IWpsConsts.KEY_APP_ID, wpsProperties.getAppId());
        paramMap.put(IWpsConsts.KEY_FILE_ID, fileId);
        paramMap.put(IWpsConsts.KEY_TOKEN_TYPE, "1");
        String signature = WpsUtil.getSignature(paramMap, wpsProperties.getAppSecret());
        wpsUrl += WpsUtil.getUrlParam(paramMap) + "&" + IWpsConsts.KEY_SIGNATURE + "=" + signature;
        WpsTokenBO wpsTokenBO = new WpsTokenBO();
        wpsTokenBO.setWpsUrl(wpsUrl);
        wpsTokenBO.setToken(wpsPermissionEnum.getValue() + ":" + userId);
        return wpsTokenBO;
    }

    @Override
    public WpsSaveFileInfoDTO save(String wpsToken, String fileId, MultipartFile file) {
        /**
        if (!file.isEmpty()) {
            try {
                BufferedOutputStream out = new BufferedOutputStream(
                        new FileOutputStream(new File()));
                out.write(file.getBytes());
                out.flush();
                out.close();
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        //// TODO: 返回保存后的文件信息 特别是文件大小与版本信息要准确
        FileModel filemodel = new FileModel();
        filemodel.name = fileNameMap.get(fileid);
        filemodel.size = file.getSize();
        filemodel.version = filemodel.version + 1;
        return filemodel;
         **/
        return null;
    }

    @Override
    public WpsFileInfoDTO getFileInfo(String wpsToken, String fileId) {
        String[] tokenArray = wpsToken.split(":");
        if (tokenArray.length != 2) {
            throw new ParamsException("token异常");
        }
        String permission = tokenArray[0];
        String userId = tokenArray[1];
        UserDTO userDTO = userBaseService.get(userId);
        if (userDTO == null) {
            throw new SearchException("用户不存在");
        }
        // 查询文件
        FilePO filePO = fileV2Service.getPO(fileId);
        if (filePO == null) {
            throw new SearchException("文件不存在");
        }
        if (!WpsUtil.isWpsFile(filePO.getFileType())) {
            throw new SearchException("文件格式不支持");
        }

        // 文件情况
        Long gmtModified = DateTime.parse(filePO.getGmtModified(), DateTimeFormat.forPattern(ISystemConstant.DATE_FORMATTER_YYYY_MM_DD_HH_MM_SS)).getMillis();
        WpsFileInfoDTO.File file = new WpsFileInfoDTO.File();
        file.setId(filePO.getFileId());
        file.setName(filePO.getFileName());
        // 按修改时间确定版本
        file.setVersion((int) (gmtModified / 1000));
        file.setSize(filePO.getFileSize());
        file.setCreator(filePO.getCreator());
        file.setCreate_time(DateTime.parse(filePO.getGmtCreate(), DateTimeFormat.forPattern(ISystemConstant.DATE_FORMATTER_YYYY_MM_DD_HH_MM_SS)).getMillis() / 1000L);
        file.setModifier(filePO.getModifier());
        file.setModify_time(gmtModified / 1000);
        file.setPreview_pages(5000);
        file.setDownload_url(serverProperties.getUrl() + "/wps/download/" + filePO.getFileId());

        // 用户情况
        WpsFileInfoDTO.User user = new WpsFileInfoDTO.User();
        user.setId(userDTO.getUserUsername());
        user.setName(userDTO.getUserName());
        user.setAvatar_url(userDTO.getUserAvatar());
        if (StringUtils.equals(permission, WpsPermissionEnum.WRITE.getValue())) {
            user.setPermission(WpsPermissionEnum.WRITE.getValue());
        } else {
            user.setPermission(WpsPermissionEnum.READ.getValue());
        }

        WpsFileInfoDTO wpsFileInfoDTO = new WpsFileInfoDTO();
        wpsFileInfoDTO.setFile(file);
        wpsFileInfoDTO.setUser(user);
        return wpsFileInfoDTO;
    }

    @Override
    public ResponseEntity<byte[]> download(String fileId) throws IOException {
        FilePO filePO = fileV2Service.getPO(fileId);
        if (filePO == null) {
            throw new SearchException("文件不存在");
        }
        if (!WpsUtil.isWpsFile(filePO.getFileType())) {
            throw new SearchException("文件格式不支持");
        }
        File file = new File(filePO.getFilePath());
        InputStream inputStream = new FileInputStream(file);
        byte[] body = new byte[inputStream.available()];
        HttpHeaders headers = new HttpHeaders();
        headers.add("Content-Disposition", "attachment;filename=" + URLEncoder.encode(filePO.getFileName(), "utf-8"));
        inputStream.read(body);
        return new ResponseEntity(body, headers, HttpStatus.OK);
    }
}
